#!/usr/bin/perl
#
# install.pl: installs webdev server scripts distribution.

# Install the scripts to support webdev-based projects, and optionally install
# the WebDev project type support (as opposed to local files and WebDAV).

$defaultphpext = 'php3';

print "WebDev Server Script Installation\n";

# Get info from version file
$WEBDEV_VERSION = GetVersionInfo();
print "Looking for WebDev installation v$WEBDEV_VERSION... ";

# Check for necessary files in current dir
$RUN_TRIAL_ZIP = "webdev_runt-$WEBDEV_VERSION.tar.gz";
$RUN_COM_ZIP = "webdev_runc-$WEBDEV_VERSION.tar.gz";
$WEB_ZIP = "webdev_web-$WEBDEV_VERSION.tar.gz";

if (-e $RUN_COM_ZIP) {
	print "found Commercial Version\n";
	$RUN_ZIP = $RUN_COM_ZIP;
} elsif (-e $RUN_TRIAL_ZIP) {
	print "found Trial Version\n";
	$RUN_ZIP = $RUN_TRIAL_ZIP;
} else {
	die "couldn't find WebDev server runtime scripts";
}
die "Couldn't find WebDev web scripts" unless -e $WEB_ZIP;
print "\n";

# Check for tar/gzip



# Get user install options
print "INSTALLATION OPTIONS\n";
print "====================\n\n";
print <<END;
To install the WebDev server scripts you need to specify a number of
options. First of all you need to specify the directory in which you
want to install the WebDev scripts which are needed by all WebDev
projects to run both with the WebDev client while they are being
edited, and also when they are used in your website.

END
ReadOption('Installation directory for WebDev runtime support',
	'/usr/local/webdev','wdinst');
print <<END;
You also need specify the directory in your web document tree where
the WebDev web support files will be installed. This is a location
that is accessible from your webserver, so that you can configure
the WebDev server scripts with a web browser, and also so that you
can use the WebDev server scripts to manage access to WebDev-type
projects in the WebDev client. (Note that if your webserver has
WebDAV support you can use this instead for project access.)

END
ReadOption('Installation directory for WebDev web support',
	'/www/webdev','webdir');
$weburi = $webdir;
$weburi =~ s#.*(/[^/]*)$#$1#;
print <<END;
Now specify the URI for the directory you have just given, i.e. the
part of the URL which would access this location, minus the server
portion (e.g. if the URL was http://www.somewhere.com/webdev then
the URI would be /webdev).

END
ReadOption('URI which WebDev web support directory maps to',
	$weburi,'weburi');
ReadOption('File extension for PHP3 files on webserver',
	$defaultphpext,'phpext');
$allowpath = $webdir;
$allowpath =~ s#/[^/]*$##;
ReadOption('Directory path allowed for WebDev-type projects',
	$allowpath,'allowpath');

$wdinc = "$wdinst/include";
$projinc = "$webdir/include";
$baseuri = $weburi unless $baseuri;

print "INSTALLING WEBDEV FILES\n";
print "=======================\n\n";
print "WebDev web installation: $webdir\n";
print "WebDev web installation URI: $weburi\n";
print "WebDev runtime installation: $wdinst\n";
print "Do you want to install the WebDev server scripts? [Y/N] ";
$yn = <STDIN>;
chop($yn);
exit unless ($yn =~ /^[Yy]/);

print "\n";
print "COPYING WEBDEV FILES\n";
print "====================\n\n";
print "Installing web files in $webdir (URI $weburi)...";
mkdir $webdir, 0755;
system "tar xfz $WEB_ZIP -C $webdir";
print "\n";
print "Installing runtime files in $wdinst...";
mkdir $wdinst, 0755;
system "tar xfz $RUN_ZIP -C $wdinst";
print "\n";
#print "PHP files have the file extension .$phpext\n";
#print "WebDev Include directory $wdinc\n";
#print "Project Include directory $projinc\n";
#print "WebDev-type projects allowed in $allowpath\n";

# Now set the PHP file extension on the files if its not the default.
# Also need to write it into index.php3?
print "Setting PHP file extension to .$phpext...";
RenameFiles($webdir,$defaultphpext,$phpext) unless $phpext eq $defaultphpext;
RenameFiles($wdinst,$defaultphpext,$phpext) unless $phpext eq $defaultphpext;
print "\n";

print "Writing config file $webdir/config.$phpext...";
WriteConfig("$webdir/config.$phpext",$baseuri,$phpext,
	$wdinc,$projinc,$allowpath);
print "\n\n";



$host = $ENV{'HOSTNAME'};
print "WEBDEV CLIENT SETTINGS\n";
print "======================\n";
print "Default WebDev URL: http://$host$weburi/webdev.$phpext\n";
print "Default Path to WebDev Libraries: $wdinc\n";

sub GetVersionInfo {
	my $ver;

	open(VFILE, "<VERSION" || die "Couldn't find VERSION file");
	$ver = <VFILE>;
	close(VFILE);
	chop($ver);
	return $ver;
}

sub ReadOption {
	my ($prompt,$default,$var) = @_;

	#print "$prompt [$default]: ";
	print "$prompt\n";
	print "\t[$default]: ";
	$$var = <STDIN>;
	chop($$var);
	$$var = $default unless length($$var) > 0;
	print "\n";
	#print "$var = $$var\n";
}

sub WriteConfig {
	my $filename = shift;
	my ($baseuri,$phpext,$wdinc,$projinc,$allowpath) = @_;

	open(CONFILE, ">$filename") || die "Couldn't open config file";
	print CONFILE "<?\r\n\$WEBDEV_CONF = array (\r\n";
	print CONFILE "\t\"baseuri\" => \"$baseuri\",\r\n";
	print CONFILE "\t\"phpext\" => \"$phpext\",\r\n";
	print CONFILE "\t\"Includes\" => array (\r\n";
	print CONFILE "\t\t\"WebDev\" => \"$wdinc\",\r\n";
	print CONFILE "\t\t\"Project\" => \"$projinc\"\r\n";
	print CONFILE "\t),\r\n\t\"Resources\" => array (\r\n";
	print CONFILE "\t\t\"Images\" => \"$baseuri/images\"\r\n";
	print CONFILE "\t),\r\n\t\"Databases\" => array (\r\n\t)\r\n);\r\n\r\n";
	print CONFILE "\$WD_INSTALL = array (\r\n";
	print CONFILE "\t\"Copyright\" => \"1997-1999 DMC Ltd\",\r\n";
	print CONFILE "\t\"Email_Contact\" => \"webdev\@dmcsoft.com\",\r\n";
	print CONFILE "\t\"WebDevVersion\" => \"$WEBDEV_VERSION\",\r\n";
	print CONFILE "\t\"AllowPaths\" => array (\r\n";
	print CONFILE "\t\t\"$allowpath\" => 1\r\n\t),\r\n";
	print CONFILE "\t\"AllowBrowsers\" => array (\r\n";
	print CONFILE "\t\t\"WebDev\" => 1\r\n\t)\r\n";
	print CONFILE ");\r\n?>\r\n";
	close(CONFILE);
}

sub RenameFiles {
	my ($dir,$fromext,$toext) = @_;
	my @entries;
	my $file,$newfile;

	# Rename all fromext files in dir to new toext
	#print "$dir\n";
	
	opendir(DIR,"$dir");
	@entries = readdir(DIR);
	closedir(DIR);
	foreach $file (@entries) {
		next if $file =~ /^\./;
		if (-f "$dir/$file" && $file =~ /$fromext$/) {
			$newfile = $file;
			$newfile =~ s/$fromext$/$toext/;
			#print "Renaming $dir/$file to $newfile\n";
			rename "$dir/$file", "$dir/$newfile";
		} elsif (-d "$dir/$file") {
			RenameFiles("$dir/$file",$fromext,$toext);
		}
	}
}

